<?php

if (!defined('_PS_VERSION_')) {
    exit();
}

/**
 * Handles Refunds and other API requests such as void.
 * @since 1.0.0
 */
class FiservApiHandler {

    /** @var string API Username */
    public static $api_username;

    /** @var string API Password */
    public static $api_password;

    /** @var string API URL */
    public static $api_url;

    /** @var string API Key Password */
    public static $certificate_key_password;

    /** @var string API Server Trust PEM */
    public static $server_trust_pem;

    /** @var string API Client Certificate PEM */
    public static $client_certificate_pemfile;

    /** @var string API Client Certificate Key */
    public static $client_certificate_keyfile;

    public static function get_refund_request($request_param) {

        $request = array(
            'URL' => self::$api_url,
            'USER' => self::$api_username,
            'PWD' => self::$api_password,
            'KEY_PWD' => self::$certificate_key_password,
            'TRUST_PEM' => self::$server_trust_pem,
            'CERT_PEM' => self::$client_certificate_pemfile,
            'CERT_KEY' => self::$client_certificate_keyfile,
            'ORDERID' => $request_param['order_hash_id'],
            'TRANSACTIONID' => $request_param['transaction_id'],
            'METHOD' => $request_param['method'],
        );


        $request_param['amount'] = array_reduce($request_param['productList'], function($accumulator, $item){
            return $accumulator + $item['total_refunded_tax_incl'];
        });

        if (!is_null($request_param['amount'])) {
            $request['AMT'] = number_format($request_param['amount'], 2, '.', '');
            $request['CURRENCYCODE'] = $request_param['order_currency'];
        }

        if ($request_param['method'] == "return") {
            $request['XML'] = '<?xml version="1.0" encoding="utf-8"?>' .
                    '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:v1="http://ipg-online.com/ipgapi/schemas/v1" xmlns:ipg="http://ipg-online.com/ipgapi/schemas/ipgapi">' .
                    '<soap:Body>' .
                    '<ipgapi:IPGApiOrderRequest
                    xmlns:v1="http://ipg-online.com/ipgapi/schemas/v1"
                    xmlns:ipgapi="http://ipg-online.com/ipgapi/schemas/ipgapi">
                    <v1:Transaction>
                    <v1:CreditCardTxType>
                    <v1:Type>return</v1:Type>
                    </v1:CreditCardTxType>
                    <v1:Payment>
                    <v1:ChargeTotal>' . $request['AMT'] . '</v1:ChargeTotal>
                    <v1:Currency>' . $request['CURRENCYCODE'] . '</v1:Currency>
                    </v1:Payment>
                    <v1:TransactionDetails>
                    <v1:OrderId>' . $request['ORDERID'] . '</v1:OrderId>
                    </v1:TransactionDetails>
                    </v1:Transaction>
                    </ipgapi:IPGApiOrderRequest>' .
                    '</soap:Body>' .
                    '</soap:Envelope>';
        } else if ($request_param['method'] == "void") {
            $request['XML'] = '<?xml version="1.0" encoding="utf-8"?>' .
                    '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:v1="http://ipg-online.com/ipgapi/schemas/v1" xmlns:ipg="http://ipg-online.com/ipgapi/schemas/ipgapi">' .
                    '<soap:Body>' .
                    '<ipgapi:IPGApiOrderRequest
                    xmlns:v1="http://ipg-online.com/ipgapi/schemas/v1"
                    xmlns:ipgapi="http://ipg-online.com/ipgapi/schemas/ipgapi">
                    <v1:Transaction>
                    <v1:CreditCardTxType>
                    <v1:Type>void</v1:Type>
                    </v1:CreditCardTxType>
                    <v1:TransactionDetails>
                    <v1:IpgTransactionId>' . $request['TRANSACTIONID'] . '</v1:IpgTransactionId>
                    </v1:TransactionDetails>
                    </v1:Transaction>
                    </ipgapi:IPGApiOrderRequest>' .
                    '</soap:Body>' .
                    '</soap:Envelope>';
        }
        return $request;
    }

    /**
     * Refund an order via Fiserv.
     * @param  float    $amount
     * @param  string   $reason
     * @return object Either an object of name value pairs for a success, or a WP_ERROR object.
     */
    public static function refund_transaction($request_param) 
    {
        $request = self::get_refund_request($request_param);
        require_once( dirname(__FILE__) . '/FiservRequest.php' );
        $process_request = new FiservRequest();
        $api_response = $process_request->build_request($request);
        return $api_response;
    }

    /**
     * Returns an array of languages accepted by the Fiserv payment platform.
     *
     * @return array[string][string]
     */
    public static function getSupportedLanguages() {
        return array(
            'de' => 'German', 'en' => 'English', 'zh' => 'Chinese', 'es' => 'Spanish', 'fr' => 'French',
            'it' => 'Italian', 'ja' => 'Japanese', 'nl' => 'Dutch', 'pl' => 'Polish', 'pt' => 'Portuguese',
            'ru' => 'Russian', 'sv' => 'Swedish', 'tr' => 'Turkish'
        );
    }

    /**
     * Returns true if the entered language (ISO code) is supported.
     *
     * @param string $lang
     * @return boolean
     */
    public static function isSupportedLanguage($lang) {
        foreach (array_keys(self::getSupportedLanguages()) as $code) {
            if ($code == Tools::strtolower($lang)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Return the list of currencies recognized by the Fiserv platform.
     *
     * @return array[int][FiservCurrency]
     */
    public static function getSupportedCurrencies() {
        $currencies = array(
            array('ARS', '032', 2), array('AUD', '036', 2), array('KHR', '116', 0), array('CAD', '124', 2),
            array('CNY', '156', 1), array('HRK', '191', 2), array('CZK', '203', 2), array('DKK', '208', 2),
            array('EKK', '233', 2), array('HKD', '344', 2), array('HUF', '348', 2), array('ISK', '352', 0),
            array('IDR', '360', 0), array('JPY', '392', 0), array('KRW', '410', 0), array('LVL', '428', 2),
            array('LTL', '440', 2), array('MYR', '458', 2), array('MXN', '484', 2), array('NZD', '554', 2),
            array('NOK', '578', 2), array('PHP', '608', 2), array('RUB', '643', 2), array('SGD', '702', 2),
            array('ZAR', '710', 2), array('SEK', '752', 2), array('CHF', '756', 2), array('THB', '764', 2),
            array('GBP', '826', 2), array('USD', '840', 2), array('TWD', '901', 1), array('RON', '946', 2),
            array('TRY', '949', 2), array('XOF', '952', 0), array('BGN', '975', 2), array('EUR', '978', 2),
            array('PLN', '985', 2), array('BRL', '986', 2)
        );

        $Fiserv_currencies = array();

        foreach ($currencies as $currency) {
            $Fiserv_currencies[] = new FiservCurrency($currency[0], $currency[1], $currency[2]);
        }

        return $Fiserv_currencies;
    }

}